declare function compress(data: ReadonlyArray<0 | 1>): Uint8Array;
declare function decompress(data: Uint8Array): Array<0 | 1>;

interface IHuffmanNode {
    value?: string;
    left?: IHuffmanNode;
    right?: IHuffmanNode;
}
type IHuffmanEncodedData = Array<0 | 1>;
type IHuffmanEncodingTable = Record<string, IHuffmanEncodedData>;
/**
 * Create a Huffman tree from the specified text.
 * @param text
 * @returns
 */
declare function fromText(text: string): IHuffmanNode;
/**
 * Build a huffman tree based on HuffmanEncodingTable.
 * @param table
 * @returns
 */
declare function fromEncodingTable(table: IHuffmanEncodingTable): IHuffmanNode;
/**
 * Build a HuffmanEncodingTable based on Huffman tree.
 * @param tree
 * @returns
 */
declare function toEncodingTable(tree: IHuffmanNode): IHuffmanEncodingTable;

/**
 * Encode texts through Huffman encode algorithm.
 * @param plaintext
 * @returns
 */
declare function encode(plaintext: string): {
    encodedData: IHuffmanEncodedData;
    encodingTable: IHuffmanEncodingTable;
    tree: IHuffmanNode;
};
/**
 * @param encodedData
 * @param tree
 * @returns
 */
declare function decode(encodedData: IHuffmanEncodedData, tree: IHuffmanNode): string;

declare const _default: {
    encode: typeof encode;
    decode: typeof decode;
    compress: typeof compress;
    decompress: typeof decompress;
    fromText: typeof fromText;
    fromEncodingTable: typeof fromEncodingTable;
    toEncodingTable: typeof toEncodingTable;
};

export { IHuffmanEncodedData, IHuffmanEncodingTable, IHuffmanNode, compress, decode, decompress, _default as default, encode, fromEncodingTable, fromText, toEncodingTable };
